﻿using ALFAT;
using System;
using System.Linq;
using System.IO.Ports;
using System.Windows;
using System.Windows.Documents;
using System.Windows.Media;

namespace ALFAT.Explorer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private CommunicationsBus alfat;

        // this is a hack to quickly implement (search for 20140623)
        // browsing app with no connections
        private bool controlsActive = false;

        public MainWindow()
        {
            this.InitializeComponent();

            this.alfat = null;
            
            if (!this.PopulateComPorts())
            {
                MessageBox.Show("There was no ALFAT Eval detected.");
            
                //this is a quick hack to allow the explorer program
                //to be looked at even when there is no Evaluation board attached
                //20140623
                this.fileTab.IsEnabled = true;
                this.miscTab.IsEnabled = true;
                this.controlsActive = false;

            }
        }

        private bool PopulateComPorts()
        {
            var ports = SerialPort.GetPortNames();
            if (ports.Length == 0)
                return false;

            this.ComPortListCmb.Items.Add("Select a port");

            foreach (string port in ports)
                this.ComPortListCmb.Items.Add(port);

            this.ComPortListCmb.SelectedIndex = 0;

            return true;
        }

        private void ShowOutput(string line, Color color)
        {
            // 20140623 so commands are still echoed, don't return if (this.controlsActive == false) 
                
            this.OutputTxt.Inlines.Add(new Run(line + "\n") { Foreground = new SolidColorBrush(color) });

            this.OutputScroll.ScrollToBottom();
        }

        private string WriteReadLine(string command)
        {
            this.ShowOutput(command, Colors.Black);
            if (this.controlsActive == true)
                this.alfat.WriteLine(command);

            return this.ReadAndShowLine();
        }

        private string WriteRead(string command)
        {
            this.ShowOutput(command, Colors.Black);

            if (this.controlsActive == true)
                this.alfat.Write(command);

            return this.ReadAndShowLine();
        }

        private string ReadAndShowLine()
        {
            string line;

            if (this.controlsActive == false) return "";
            try
            {
                line = this.alfat.ReadLine();
            }
            catch
            {
                return "";
            }

            var color = Colors.Green;

            if (   line.StartsWith("!") 
			    && line != "!00"
				&& line != "!04" )
            {
                color = Colors.Red;
                // line += "\n";
            }


            this.ShowOutput(line, color);
            return line;
        }

        private void EndOfCommand(){
            this.ShowOutput("\n",(Color) Colors.Black);
        }
        

        private void DisableSDReader()
        {
            this.sdReaderChk.IsChecked = false;
            this.sdReaderHighChk.IsChecked = false;
        }

        private void EnableSDReader()
        {
            this.sdCardChk.IsChecked = false;
            this.usb1Chk.IsChecked = false;
            this.usb1HighChk.IsChecked = false;
        }

        private void CloseBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Prompt("Enter Handle of file to close:", "0");
            if (dialog.ShowDialog().Value)
            {
                this.WriteReadLine("C " + dialog.ResponseText);
                EndOfCommand();
            }
        }

        private void DeleteBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Prompt("Delete file or directory:", "M:\\oldfile.txt");
            if (dialog.ShowDialog().Value)
            {
                this.WriteReadLine("D " + dialog.ResponseText);
                EndOfCommand();
            }
        }

        private void OpenBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Open("0", "M:\\TEST.TXT", "R");
            if (dialog.ShowDialog().Value)
            {
                this.WriteReadLine("O " + dialog.Handle + dialog.AccessMode + ">" + dialog.FileName);
                EndOfCommand();
            }
        }

        private void SeekBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Prompt("Enter Handle (0 to F):", "0");
            if (dialog.ShowDialog().Value)
            {
                string handle = dialog.ResponseText;

                dialog = new Prompt("To Position (as Hex):", "0");
                if (dialog.ShowDialog().Value)
                {
                    this.WriteReadLine("P " + handle + ">" + dialog.ResponseText);
                    EndOfCommand();
                }
            }
        }

        private void ReadBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Read("0", "A");
            if (dialog.ShowDialog().Value)
            {
                this.WriteReadLine("R " + dialog.Handle + "F>" + dialog.Count);
                this.ReadAndShowLine(); //data + "$"{bytes read in hexidecimal}
                this.ReadAndShowLine(); //final result stribng
                EndOfCommand();
            }
        }

        private void TellBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Prompt("Enter Handle (0 to F):", "0");
            if (dialog.ShowDialog().Value)
            {
                this.WriteReadLine("Y " + dialog.ResponseText);
                this.ReadAndShowLine(); //position
                this.ReadAndShowLine(); //final result stribng
                EndOfCommand();
            }
        }

        private void WriteBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Write("0", "A", "1234567890");
            if (dialog.ShowDialog().Value)
            {
                int length = int.Parse(dialog.Count, System.Globalization.NumberStyles.HexNumber);

                if (dialog.Data.Length < length)
                    dialog.Count = dialog.Data.Length.ToString("X");

                this.WriteReadLine("W " + dialog.Handle + ">" + dialog.Count);
                this.WriteRead(dialog.Data);
                this.ReadAndShowLine(); //final result string
                EndOfCommand();
            }
        }

        private void GetDateTimeBtn_Click(object sender, RoutedEventArgs e)
        {
            this.WriteReadLine("G T");
            this.ReadAndShowLine();
            
            this.WriteReadLine("G D");
            this.ReadAndShowLine();
            EndOfCommand();
        }

        private void ShowFilesBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Prompt("Enter Full path of directory", "M:\\");
            if (dialog.ShowDialog().Value)
            {
                this.WriteReadLine("@ " + dialog.ResponseText.Trim());
                
                while (true)
                {
                    var rs = this.WriteReadLine("N");
                    if (rs != "!00") 
                        break;

                    this.ReadAndShowLine(); //name
                    this.ReadAndShowLine(); //attributes
                    this.ReadAndShowLine(); //size
                    this.ReadAndShowLine(); //final result
                }
                EndOfCommand();
            }
        }

        private void StatusRegistersBtn_Click(object sender, RoutedEventArgs e)
        {
            for (int reg = 0; reg < 3; reg++)
            {
                this.WriteReadLine("J " + reg);
                this.ReadAndShowLine();
                this.ReadAndShowLine();
            }
            EndOfCommand();
        }

        private void BatteryRTCBtn_Click(object sender, RoutedEventArgs e)
        {
            this.WriteReadLine("T S");
            EndOfCommand();
        }

        private void InternalRTCBtn_Click(object sender, RoutedEventArgs e)
        {
            this.WriteReadLine("T B");
            EndOfCommand();
        }

        private void SetDateTimeBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new Prompt("Enter Time and Date in FAT format", "34212002");
            if (dialog.ShowDialog().Value)
            {
                this.WriteReadLine("S " + dialog.ResponseText.Trim());
                EndOfCommand();
            }
        }

        private void ClearOutputBtn_Click(object sender, RoutedEventArgs e)
        {
            this.OutputTxt.Inlines.Clear();
        }

        private void CustomWriteBtn_Click(object sender, RoutedEventArgs e)
        {
            this.ShowOutput(this.customCommandTxt.Text, Colors.Black);

            if (this.controlsActive == true)
                this.alfat.WriteLine(this.customCommandTxt.Text);
        }

        private void CustomReadBtn_Click(object sender, RoutedEventArgs e)
        {
            this.ReadAndShowLine();
        }

        private void ComPortListCmb_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            var port = this.ComPortListCmb.SelectedItem.ToString();

            if (!port.StartsWith("COM"))
                return;

            if (this.alfat != null)
                this.alfat.Dispose();

            this.alfat = new CommunicationsBus(port);

            this.sdCardChk.IsEnabled = true;
            this.sdReaderChk.IsEnabled = true;
            this.sdReaderHighChk.IsEnabled = true;
            this.usb0Chk.IsEnabled = true;
            this.usb1Chk.IsEnabled = true;
            this.usb1HighChk.IsEnabled = true;
            this.customReadBtn.IsEnabled = true;
            this.customWriteBtn.IsEnabled = true;

            this.fileTab.IsEnabled = true;
            this.miscTab.IsEnabled = true;
            this.controlsActive = true;
            MessageBox.Show("Device mounting can be slow. Be patient for result in the Console panel.");
        }

      

        private void SDCardChk_Checked(object sender, RoutedEventArgs e)
        {
            this.DisableSDReader();

            if (this.WriteReadLine("I M:") != "!00")
                this.sdCardChk.IsChecked = false;
            EndOfCommand();
        }

        private void USB0Chk_Checked(object sender, RoutedEventArgs e)
        {
            if (this.WriteReadLine("I U0:") != "!00")
                this.usb0Chk.IsChecked = false;
            EndOfCommand();
        }

        private void USB1Chk_Checked(object sender, RoutedEventArgs e)
        {
            this.DisableSDReader();
            this.usb1HighChk.IsChecked = false;

            if (this.WriteReadLine("I U1:") != "!00")
                this.usb1Chk.IsChecked = false;
            EndOfCommand();
        }

        private void USB1HighChk_Checked(object sender, RoutedEventArgs e)
        {
            this.DisableSDReader();
            this.usb1Chk.IsChecked = false;

            if (this.WriteReadLine("I U1:H") != "!00")
                this.usb1HighChk.IsChecked = false;
            EndOfCommand();
        }

        private void SDReaderChk_Checked(object sender, RoutedEventArgs e)
        {
            this.EnableSDReader();
            this.sdReaderHighChk.IsChecked = false;

            if (this.WriteReadLine("I D:") != "!00")
                this.sdReaderChk.IsChecked = false;
            EndOfCommand();
        }

        private void SDReaderHighChk_Checked(object sender, RoutedEventArgs e)
        {
            this.EnableSDReader();
            this.sdReaderChk.IsChecked = false;

            if (this.WriteReadLine("I D:H") != "!00")
                this.sdReaderHighChk.IsChecked = false;
            EndOfCommand();
        }

        private void AboutMnu_Click(object sender, RoutedEventArgs e)
        {
            new About().ShowDialog();
        }
    }
}
